/******************************************************************************\
 * File: cbuffer.h                                                            *
 * Author: James Greensky                                                     *
 * Notes: Declaration of a circular byte buffer.                              *
\******************************************************************************/

#ifndef cbuffer_h
#define cbuffer_h

#include "logger.h"

struct CBufferRecord;
typedef struct CBufferRecord* CBuffer;

inline int CBufferSize(CBuffer buffer);
inline int CBufferCapacity(CBuffer buffer);
CBuffer CBufferCreate(int capacity, int itemSize);
void CBufferDispose(CBuffer buffer);
void CBufferClear(CBuffer buffer);
void* CBufferGetWriteBlock(CBuffer buffer, int* count);
void* CBufferGetWriteBlockAbs(CBuffer buffer, int count);
int CBufferCommitBlock(CBuffer buffer, int count);
void* CBufferGetReadBlock(CBuffer buffer, int* count);
void* CBufferGetReadBlockAt(CBuffer buffer, int offset, int* count);
int CBufferDecommitBlock(CBuffer buffer, int count);
int CBufferWrite(CBuffer buffer, void* buf, int count);
int CBufferRead(CBuffer buffer, void* buf, int count);
int CBufferPeek(CBuffer buffer, void* buff, int count);
int CBufferPeekAt(CBuffer buffer, void* buff, int offset, int count);
int CBufferInsert(CBuffer buffer, void* buff, int count);
int CBufferInsertBuffer(CBuffer dest, CBuffer src);
int CBufferCopyBuffer(CBuffer dest, CBuffer src);
int CBufferRemove(CBuffer buffer, int count);
int CBufferRemoveAt(CBuffer buffer, int offset, int count);

#endif
