/******************************************************************************\
 * Name: hvr_cl.c                                                             *
 * Author: James Greensky                                                     *
 * Notes: mock hvr_cl program                                                 *
\******************************************************************************/

#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/uio.h>
#include <fcntl.h>
#include <unistd.h>
#include <math.h>
#include <signal.h>

static int Closing = 0;

static void SIGTERM_Handler(int signal) {
  Closing = 1;
}

int main(int argc, char** argv) {
  int total = 245760;
  char image[4][total];
  char buffer[4096];
  char *size, *ptr, *hvfile;
  char delimiter[] = " \t\r\n,;x";
  char whitespace[] = " \t\r\n";
  int retVal = 0, k = -1, j, index, width = 320, xpics, height = 256,status = 0, vecs, count, length;
  struct iovec *vec, *vptr;
  struct sigaction sa;

  memset(&sa, 0, sizeof(sa));
  sa.sa_handler = &SIGTERM_Handler;
  while (sigaction(SIGTERM, &sa, NULL) == -1) {
    if (errno != EINTR) {
      retVal = errno;
      fprintf(stderr, "%s: sigaction failed for SIGTERM.  %s.\n\n", 
	      program_invocation_short_name, strerror(retVal));
      exit(2);
    }
  }  
  while (sigaction(SIGINT, &sa, NULL) == -1) {
    if (errno != EINTR) {
      retVal = errno;
      fprintf(stderr, "%s: sigaction failed for SIGINT.  %s.\n\n", 
	      program_invocation_short_name, strerror(retVal));
      exit(2);
    }
  }  
  while (sigaction(SIGQUIT, &sa, NULL) == -1) {
    if (errno != EINTR) {
      fprintf(stderr, "%s: sigaction failed for SIGQUIT.  %s.\n\n", 
	      program_invocation_short_name, strerror(retVal));
      exit(2);
    }
  }  
  while (sigaction(SIGHUP, &sa, NULL) == -1) {
    if (errno != EINTR) {
      fprintf(stderr, "%s: sigaction failed for SIGHUP.  %s.\n\n", 
	      program_invocation_short_name, strerror(retVal));
      exit(2);
    }
  }

  for (index = 0; index < 4; index++) {
    switch (index) {
    case 0:
      k = open("tl.raw", O_RDONLY);
      break;
    case 1:
      k = open("tr.raw", O_RDONLY);
      break;
    case 2:
      k = open("bl.raw", O_RDONLY);
      break;
    case 3:
      k = open("br.raw", O_RDONLY);
    }
    if (k == -1) {
      retVal = errno;
      fprintf(stderr, "open failed.  %s\n", strerror(retVal));
      return 1;
    }
    j = 0;
    while (j < 245760) {
      retVal = read(k, &image[index][j], 245760 - j);
      if (retVal == -1) {
	retVal = errno;
	if (retVal == EINTR)
	  continue;
	fprintf(stderr, "read failed.  %s\n", strerror(retVal));
	return 2;
      }
      j += retVal;
    }
  }
  
  hvfile = malloc(sizeof(char));
  hvfile[0] = '\0';
  index = 0;
  for(;!Closing;) {
    while (fgets(buffer, 4096, stdin) == NULL) {
      k = errno;
      if (k != EINTR) {
	fprintf(stderr, "fgets failed.  %s\n", strerror(k));
	free(hvfile);
	return 3;
      }
      if (Closing) {
	free(hvfile);
	exit(0);
      }
    }
    if (strncmp(buffer, "render", 6) == 0) {
      if (status)
	write(STDOUT_FILENO, &status, 4);
      else {
	xpics = (int)ceil(width/320.0);
	vecs = height * xpics + 1;
	vptr = malloc(sizeof(struct iovec) * vecs);
	if (vptr == NULL) {
	  status = 1;
	  write(STDOUT_FILENO, &status, 4);
	}
	else {
	  vec = vptr;
	  vec->iov_base = &status;
	  vec->iov_len = 4;
	  vec++;
	  length = 0;
	  for (k = 0; k < height; k++) {
	    for (j = 0; j < xpics; j++) {
	      vec->iov_base = &image[index][(320 * (k % 256)) * 3];
	      retVal = width - (320 * j);
	      vec->iov_len = (retVal > 320) ? (320 * 3) : (retVal * 3);
	      length += vec->iov_len;
	      vec++;
	    }
	  }
	  j = 0;
	  count = 0;
	  vec = vptr;
	  while (j < length) {
	    j += writev(STDOUT_FILENO, vec, vecs - count);
	    if (j < length) {
	      for (vec = vptr, k = 0, count = 0; k < length; vec++, count++)
		k += vec->iov_len;
	    }
	  }
	  free(vptr);
	}
      }
      status = 0;
      width = 320;
      height = 256;
    }
    else if (strncmp(buffer, "imagesize", 9) == 0) {
      k = 0;
      size = strtok(&buffer[9], delimiter);
      while (size != NULL && k < 2) {
	if (strlen(size)) {
	  errno = 0;
	  retVal = strtol(size, &ptr, 10);
	  if (errno == ERANGE || retVal > 4096 || retVal < 1 || ptr == size || *ptr != '\0') {
	    status = 1;
	    k = 2;
	  }
	  else {
	    if (k)
	      height = retVal;
	    else
	      width = retVal;
	    k++;
	  }
	}
	size = strtok(NULL, delimiter);
      }
    }
    else if (strncmp(buffer, "hvfile", 6) == 0) {
      while (!(k = strlen(ptr = strtok(&buffer[6], whitespace))));
      if (strncmp(ptr, hvfile, k)) {
	free(hvfile);
	hvfile = strdup(ptr);
	index = (index + 1) % 4;
      }
    }
    else if (strncmp(buffer, "exit", 4) == 0) {
      Closing = 1;
    }
  }
  free(hvfile);
  return 0;
}
