/******************************************************************************\
 * File: logger.h                                                             *
 * Author: James Greensky                                                     *
 * Notes: Declare logging routines                                            *
\******************************************************************************/

#ifndef logger_h
#define logger_h

#include <sys/types.h>

enum LogLevels { LogOff, LogFatal, LogError, LogWarn, LogInfo, LogDebug, LogAll };
typedef enum LogLevels LogLevel;
struct LogRecord;
typedef struct LogRecord* Log;

inline int LogIsFatal(Log log);
inline int LogIsError(Log log);
inline int LogIsWarn(Log log);
inline int LogIsInfo(Log log);
inline int LogIsDebug(Log log);

Log LogCreate(const char* name);
void LogDispose(Log log);
void LogSetLevel(Log log, LogLevel logLevel);
LogLevel LogGetLevel(Log log);
char* LogGetPath(Log log);
off_t LogGetMaxSize(Log log);
void LogSetMaxSize(Log log, off_t maxSize);
int LogOpen(Log log, char* path);
void LogClose(Log log);
int LogMessage(Log log, LogLevel logLevel, const char* format, ...);
int LogFlush(Log log);

#endif
