
/******************************************************************************\
 * Name: queue.c                                                              *
 * Author: James Greensky                                                     *
 * Notes: implementation of queue data structure                              *
\******************************************************************************/

#include "queue.h"
#include <assert.h>
#include <stdlib.h>

struct QueueElementRecord;
typedef struct QueueElementRecord* QueueElement;

struct QueueElementRecord {
  void* Item;
  QueueElement Next;
};

struct QueueRecord {
  QueueElement Head;
  QueueElement Tail;
  int Size;
};

int QueueSize(Queue queue) {
  assert(queue != NULL);
  
  return queue->Size;
}

Queue QueueCreate() {
  Queue queue = malloc(sizeof(struct QueueRecord));
  if (queue == NULL)
    return NULL;
  queue->Head = malloc(sizeof(struct QueueElementRecord));
  if (queue->Head == NULL) {
    free(queue);
    return NULL;
  }
  queue->Head->Item = queue->Head->Next = NULL;
  queue->Tail = queue->Head;
  queue->Size = 0;
  return queue;
}

void QueueDispose(Queue queue) {
  assert(queue != NULL);

  QueueElement element, save;
  for (element = queue->Head; element != NULL;) {
    save = element->Next;
    free(element);
    element = save;
  } 
  
  free(queue);
}

int QueueEnqueue(Queue queue, void* item) {
  assert(queue != NULL);
  assert(item != NULL);

  QueueElement tail = malloc(sizeof(struct QueueElementRecord));
  if (tail == NULL)
    return -1;
  tail->Item = item;
  tail->Next = NULL;
  queue->Tail->Next = tail;
  queue->Tail = tail;
  queue->Size++;
  return 0;
}

void* QueueDequeue(Queue queue) {
  assert(queue != NULL);
  assert(queue->Size != 0);

  void* item = queue->Head->Next->Item;
  QueueElement head = queue->Head->Next->Next;
  free(queue->Head->Next);
  queue->Head->Next = head;
  if (head == NULL)
    queue->Tail = queue->Head;
  queue->Size--;
  return item;
}

void* QueuePeek(Queue queue) {
  assert(queue != NULL);
  assert(queue->Size != 0);
  
  return queue->Head->Next->Item;
}

void QueueRemove(Queue queue, void* item) {
  assert(queue != NULL);
  assert(item != NULL);
  
  QueueElement element, previous;
  for (previous = queue->Head, element = queue->Head->Next; element != NULL && element->Item != item; 
       previous = element, element = element->Next);
  if (element != NULL) {
    previous->Next = element->Next;
    free(element);
    if (previous->Next == NULL)
      queue->Tail = previous;
    queue->Size--;
  }
}
