/******************************************************************************\
 * File: server.h                                                             *
 * Author: James Greensky                                                     *
 * Notes: Declare server routines                                             *
\******************************************************************************/

#ifndef server_h
#define server_h
#define BUFFER_SIZE 115840
/*#define _VALGRIND*/

#include "cbuffer.h"
#include "logger.h"
#include <arpa/inet.h>

extern char* ModuleDirectory;
extern Log Logger;

typedef enum LcseClientEvents {
  ClientAccepted,
  ClientDisconnected,
  ClientDataReceived
} LcseClientEvent;

typedef enum LcseExecutableEvents {
  ExecutableExited,
  ExecutableDataReady,
  ExecutableWriteReady
} LcseExecutableEvent;

typedef enum LcseIOEvents {
  IOWriteReady,
  IOReadReady
} LcseIOEvent;

typedef enum LcseSelectTypes {
  SelectRead,
  SelectWrite
} LcseSelectType;

typedef struct LcseExecutableRecord {
  char** Argv;
  pid_t ProcessID;
  int StdIn, StdOut, Status;
}* LcseExecutable;

typedef struct LcseClientRecord {
  int Socket;
  CBuffer InBuffer, OutBuffer;
}* LcseClient;

typedef struct LcseAddressRecord {
  struct in_addr Address;
  uint16_t Port;
  int Domain;
  struct LcseAddressRecord* Next;
}* LcseAddress;

int ServerRun(LcseAddress address);
void CloseClient(LcseClient client);
pid_t SpawnExecutable(void* handle, char** argv);
int AddSelectIO(void* handle, int fd, LcseSelectType type);
void RemoveSelectIO(void* handle, int fd, LcseSelectType type);
int IsSelectIOSet(int fd, LcseSelectType type);
void AddExecutableStdIn(LcseExecutable executable);
void RemoveExecutableStdIn(LcseExecutable executable);
int SetNonBlockingIO(int fd);
int SetCloseOnExecIO(int fd);
void Cclose(int fd);

#endif
